#ifndef prosit_h
#define prosit_h

#define PROSIT_VERSION_STRING "0811"
#define PROSIT_VERSION_NUMBER 0x30383031	// "0811"

#define EVENT_WAKEUP 0x0000
#define EVENT_QUIT 0xffff
#define EVENT_HAVE_QUIT 0xfffe

#include "romcalls.h"

typedef int PLOCK;

typedef struct {
  int action;
  BYTE data[20];
} EventStruct;

typedef struct {
  int size, readoffset, writeoffset, pending;
  volatile int free, available;
  BYTE data[0];
} RBUFFER;

#ifdef TI92PLUS
  typedef union {
    WORD data;
    struct {
        unsigned int down : 1;
        unsigned int right : 1;
        unsigned int up : 1;
        unsigned int left : 1;
        unsigned int hand : 1;
        unsigned int shift : 1;
        unsigned int diamond : 1;
        unsigned int second : 1;

        unsigned int flag0_7 : 1;
        unsigned int flag0_6 : 1;
        unsigned int flag0_5 : 1;
        unsigned int flag0_4 : 1;
        unsigned int flag0_3 : 1;
        unsigned int flag0_2 : 1;
        unsigned int flag0_1 : 1;
        unsigned int flag0_0 : 1;
    } flags;
  } ControlKeysStruct;
  int _ti92plus;
#endif

#ifdef TI89
  typedef union {
    WORD data;
    struct {
        unsigned int alpha : 1;
        unsigned int diamond : 1;
        unsigned int shift : 1;
        unsigned int second : 1;
        unsigned int right : 1;
        unsigned int down : 1;
        unsigned int left : 1;
        unsigned int up : 1;

        unsigned int flag0_7 : 1;
        unsigned int flag0_6 : 1;
        unsigned int flag0_5 : 1;
        unsigned int flag0_4 : 1;
        unsigned int flag0_3 : 1;
        unsigned int flag0_2 : 1;
        unsigned int flag0_1 : 1;
        unsigned int flag0_0 : 1;
    } flags;
  } ControlKeysStruct;
  int _ti89;
#endif


#define STR(x) #x
#define XSTR(x) STR(x)
#define S2L(str) ((long)str[0] << 24) + ((long)str[1] << 16) + ((long)str[2] << 8) + ((long)str[3])
#define S2W(str) ((long)str[0] << 8) + ((long)str[1])

typedef struct {
  BYTE data;
} MEMORY_BYTE;

typedef struct {
  WORD data;
} MEMORY_WORD;

typedef struct {
  ULONG data;
} MEMORY_LONG;


//
// Prosit executable file structure
//
typedef struct {
  WORD filesize;
  WORD rts;
  WORD offset_prosit;
  ULONG signature;
  WORD reloccount;
  WORD offset_comment;
  WORD offset_main;
  WORD offset_exit;
  union {
    WORD all;
    struct {
      unsigned int ti89_emulation : 1;
      unsigned int flag1_6 : 1;
      unsigned int flag1_5 : 1;
      unsigned int flag1_4 : 1;
      unsigned int flag1_3 : 1;
      unsigned int flag1_2 : 1;
      unsigned int flag1_1 : 1;
      unsigned int copy_of_archived : 1;

      unsigned int flag0_7 : 1;
      unsigned int flag0_6 : 1;
      unsigned int flag0_5 : 1;
      unsigned int flag0_4 : 1;
      unsigned int flag0_3 : 1;
      unsigned int flag0_2 : 1;
      unsigned int runs_on_92plus : 1;
      unsigned int runs_on_89 : 1;
    } bits;
  } flags;
} PROSIT_FILE_HEADER;

typedef struct ListElementHeaderStruct {
  struct ListElementHeaderStruct *next;
  WORD id;
  signed char priority;
  BYTE flags;
} ListElementHeader;

enum ListTypes { PriorityListType = 0, FIFOListType = 1, LIFOListType = 2 };

typedef struct ListHeaderStruct {
  ListElementHeader *first,*last;
  PLOCK lock;
  int type;
} ListHeader;

typedef struct {
  ListElementHeader header;
  ListHeader list;
  int count;
} SEMAPHORE;

//
// Prosit header
//
#define PROSIT_FILETYPE_PROGRAM 0x70726f67	// "prog"
#define PROSIT_FILETYPE_BACKGROUND 0x626b6772	// "bkgr"

#define PROSIT_BEGIN_HEADER 0x02ff
#define PROSIT_STACKSIZE 0x0401
#define PROSIT_EVENTBUFFERSIZE 0x0402
#define PROSIT_VERSION 0x0601
#define PROSIT_FILETYPE 0x0602
#define PROSIT_END_HEADER 0x0000

typedef union {
  WORD fieldtype;
  struct {
    BYTE next;
    BYTE type;
  } parts;
} PrositHeaderElement_Header;

typedef struct {
  PrositHeaderElement_Header header;
  WORD stacksize;
} PrositHeaderElement_Stacksize;

typedef struct {
  PrositHeaderElement_Header header;
  WORD eventbuffersize;
} PrositHeaderElement_Eventbuffersize;

typedef struct {
  PrositHeaderElement_Header header;
  ULONG version;
} PrositHeaderElement_Version;

typedef struct {
  PrositHeaderElement_Header header;
  ULONG filetype;
} PrositHeaderElement_Filetype;

#define BEGIN_PROSIT_HEADER(name) PrositHeaderElement_Header name = {PROSIT_BEGIN_HEADER};
#define DEFINE_PROSIT_STACKSIZE(size) asm(" .word "##XSTR(PROSIT_STACKSIZE)##","##XSTR(size));
#define DEFINE_PROSIT_VERSION(version) asm(" .word "##XSTR(PROSIT_VERSION)##"\n .long "##XSTR(version));
#define DEFINE_PROSIT_FILETYPE(type) asm(" .word "##XSTR(PROSIT_FILETYPE)##"\n .long "##XSTR(type));
#define DEFINE_PROSIT_EVENTBUFFERSIZE(size) asm(" .word "##XSTR(PROSIT_EVENTBUFFERSIZE)##","##XSTR(size));
#define END_PROSIT_HEADER asm(" .word "##XSTR(PROSIT_END_HEADER));


//
// Event types
//
#define QUIT_EVENT 0x71756974		// "quit"
#define MOUSEBUTTON_EVENT 0x6d6f7562	// "moub"


//
// Prosit process context structure
//
typedef struct PROCESS_CONTEXT_ENTRY_STRUCT {
  ListElementHeader header;
  ULONG registers[15];
  void *usp;
  WORD sr;
  void *pc;
  ULONG tiosdata[9];
  union {
    WORD all;
    struct {
      unsigned int flag1_7 : 1;
      unsigned int flag1_6 : 1;
      unsigned int flag1_5 : 1;
      unsigned int flag1_4 : 1;
      unsigned int flag1_3 : 1;
      unsigned int flag1_2 : 1;
      unsigned int flag1_1 : 1;
      unsigned int flag1_0 : 1;

      unsigned int flag0_7 : 1;
      unsigned int flag0_6 : 1;
      unsigned int flag0_5 : 1;
      unsigned int flag0_4 : 1;
      unsigned int fork : 1;
      unsigned int copy_of_archived : 1;
      unsigned int in_interrupt : 1;
      unsigned int sleeping : 1;
    } bits;
  } flags;
  HANDLE filehandle;
  HANDLE stackhandle;
  HANDLE eventbufferhandle;
  HANDLE parent;
  void *prositheader;
} PROCESS_CONTEXT_ENTRY;

#define CS_INIT_NORMAL		0x0000
#define CS_INIT_FORK		0x0008
#define CS_FORK_KEEP_FLAGS	0x0004

#define ToggleScheduler() asm(" trap #13");
#define BreakAndCooperate() asm(" trap #14");
#define ToggleProcessorMode() asm(" trap #15");



//
// Functions exported by Prosit
//
#define CurrentProcessID prosit__0000
extern HANDLE CurrentProcessID;

#define Ticks prosit__0001
extern volatile ULONG Ticks;

#define CurrentKey prosit__0002
extern volatile WORD CurrentKey;

#define ControlKeys prosit__0003
extern volatile ControlKeysStruct ControlKeys;

#define InitializeLock prosit__0004
void InitializeLock(PLOCK *l);

#define Lock prosit__0005
void Lock(PLOCK *l);

#define Unlock prosit__0006
void Unlock(PLOCK *l);

#define ExecuteFile prosit__0007
void executeFile(HANDLE filehandle);

#define GetPrositHeaderOfFile prosit__0008
void *GetPrositHeaderOfFile(HANDLE filehandle);

#define GetPrositHeaderOfProcess prosit__0009
void *GetPrositHeaderOfProcess(HANDLE pid);

#define FindPrositHeaderField prosit__000A
void *FindPrositHeaderField(PrositHeaderElement_Header *current, WORD type);

#define ListInitialize prosit__000B
void ListInitialize(ListHeader *List, int type);

#define ListPrevious prosit__000C
ListElementHeader *ListPrevious(ListElementHeader *elementheader);

#define ListInsert prosit__000D
void ListInsert(ListHeader *header, ListElementHeader *elementheader);

#define ListRemove prosit__000E
void ListRemove(ListHeader *header, ListElementHeader *elementheader);

#define ListFind prosit__000F
ListElementHeader *ListFind(ListHeader *header, WORD idstring);

#define ListSize prosit__0010
int ListSize(ListHeader *header);

#define TerminateProcess prosit__0011
void TerminateProcess(HANDLE pid);

#define NewProcess prosit__0012
HANDLE NewProcess(HANDLE filehandle);

#define ForkProcess prosit__0013
HANDLE ForkProcess(void *header, void *entrypoint);

#define RunningProcesses prosit__0014
int RunningProcesses(void);

#define SleepingProcesses prosit__0015
int SleepingProcesses(void);

/*
#define Sleep prosit__0016
void Sleep(HANDLE pid);

#define WakeUp prosit__0017
void WakeUp(HANDLE pid);
*/

#define TriggerEvent prosit__0017
void TriggerEvent(HANDLE target, WORD size, void *data);

#define BroadcastEvent prosit__0018
void BroadcastEvent(WORD size, void *data);

#define PendingEvents prosit__0019
int PendingEvents(void); 

#define ReadEvent prosit__001A
HANDLE ReadEvent(void *dest);

#define RBufferInitialize prosit__001B
void RBufferInitialize(RBUFFER *buffer, WORD size);

#define RBufferWrite prosit__001C
void RBufferWrite(RBUFFER *buffer, BYTE d);

#define RBufferRead prosit__001D
BYTE RBufferRead(RBUFFER *buffer);

#define SemaphoreRegister prosit__001E
void SemaphoreRegister(SEMAPHORE *s, int start);

#define SemaphoreUnregister prosit__001F
void SemaphoreUnregister(SEMAPHORE *s);

#define SemaphoreWait prosit__0020
void SemaphoreWait(SEMAPHORE *s);

#define SemaphoreSignal prosit__0021
void SemaphoreSignal(SEMAPHORE *s);

#endif
