#ifndef PROSIT_MACROS
#define PROSIT_MACROS

#ifndef WORD
#define WORD unsigned short int
#endif //WORD

typedef struct EventStruct
{
	void* fn;
	char id[4];
} XEVENT;

typedef struct _PrositHeader
{
	char type_id[2];
	char file_type[2];
	WORD stack_size;
	XEVENT eventlist[0];
} PrositHeader;


//*******************  HEADER DECLARATION MACROS *****************
#define DECLARE_PROSIT_HEADER(stacksize) PrositHeader _prosit = { PROSIT_VERSION_STRING, PROSIT_TYPE_PROGRAM, stacksize, 
#define END_PROSIT_HEADER 	 };


#define ON_EVENT(function,eventid) { function, eventid},
#define DECLARE_EVENTS {
#define END_EVENTS {0, ""} }

#define DECLARE_FORK_HEADER(name,stacksize) PrositHeader name = { PROSIT_VERSION_STRING, PROSIT_TYPE_PROGRAM, stacksize, 
#define END_FORK_HEADER 	 };




//******************     LOCKING FUNCTIONS   ********************
/*
typedef signed int LOCK;

#define CREATE_LOCK(x) ENABLE_LOCK(LOCK x)
#define ENABLE_LOCK(x) x = -1;

void GetLock(LOCK *lock)
{
	while(++(*lock))
	{
		(*lock)--;
		Break_And_Cooperate();
	}
}

void ReleaseLock(LOCK* lock)
{
	(*lock)--;
}
*/

//************** UTILITY MACROS *********************

#ifndef TI_92
int _ti89;
#define SCREEN_WIDTH 160
#define SCREEN_HEIGHT 100
#else
int _ti92;
#define SCREEN_WIDTH 204
#define SCREEN_HEIGHT 128
#endif

//***************** GUI Object Macros ****************

/*
 * VARIABLE(name,data)
 * 
 * Assigns a name [name] to [data]
 * Must be used only inside a GUI_SECTION (*see BEGIN_GUI_SECTION) or possibly a GUI_MANAGER (*see BEGIN_GUI_MANAGER)
 * Example:
 * 	VARIABLE(myTextButton, TEXTBUTTON(10,10,40,10,0,0,"This is my Text button") )
*/
#define VARIABLE(name,vardata) "\n" #name ": " vardata

/*
 * ASM_STRING(x)
 *  
 * Used internally by GUI macros
*/
#define ASM_STRING(X) " .ascii \"" X "\\0\"\n"

/*
 * BEGIN_GUI_SECTION(varName)
 *
 * Declares a graphical GUI structure named [varName]
 * **Must be paired with an END_GUI_SECTION
 *
 * Example:
 * 	BEGIN_GUI_SECTION(myGUI)
 * 	  TEXT(0,0,0,0,"My Text Label")
 * 	  // [more objects declarations] ...
 * 	END_GUI_SECTION
*/
#define BEGIN_GUI_SECTION(name) asm("\n .even\n" #name ":\n"

/*
 * TEXT(X, Y, Font, Flags, String)
 *
 * GUI Object -- Must be declared inside a BEGIN_GUI_SECTION() / END_GUI_SECTION group
 * Displays [String] at (X,Y) using [Font] and [Flags]
 *
 * Example:
 * 	  TEXT(10,1,0,0,"This is a string")
 * 
 * TEXT_PTR(X, Y, Font, Flags, PointerToString)
 *
 * Same as TEXT(..), except you pass a pointer to a string (char*) instead of a literal string
*/
#define TEXT(x,y,font,flags,text) \
		"\n .byte 64," #x "," #y ", " #font ", " #flags "\n" ASM_STRING(text) 

#define TEXT_PTR(x,y,font,flags,ptr) \
		"\n .byte 64," #x "," #y ", " #font ", " #flags "\n .even\n .word 0\n .long " #ptr "\n" 

		
/*
 * TEXTBUTTON(X, Y, Width, Hieght, Font, Flags, String)
 *
 * GUI Object -- Must be declared inside a BEGIN_GUI_SECTION() / END_GUI_SECTION group
 * Displays a button at (X,Y) with Width and Height specified, with label [String] (centered) using [Font] and [Flags]
 *
 * Example:
 * 	  TEXTBUTTON(10,1,30,8,0,0,"This is a string")
 *
 * TEXTBUTTON_PTR(X, Y, Width, Hieght, Font, Flags, String)
 *
 * Same as TEXT(..), except you pass a pointer to a string (char*) instead of a literal string
*/
#define TEXTBUTTON(x,y,w,h,font,flags,text) \
		"\n .byte 1, " #x ", " #y ", " #w ", " #h ", " #font ", " #flags "\n" ASM_STRING(text) 
#define TEXTBUTTON_PTR(x,y,w,h,font,flags,ptr) \
		"\n .byte 1, " #x ", " #y ", " #w ", " #h ", " #font ", " #flags "\n .even\n .word 0\n .long " #ptr "\n" 

/*
 * BOX(X, Y, Width, Height, Color, Flags)
 *
 * GUI Object -- Must be declared inside a BEGIN_GUI_SECTION() / END_GUI_SECTION group
 * Draws a Box at (X,Y) with specified Width and Hieght, using [Color] and [Flags] 
 *
 * Example:
 * 	  BOX(10,1,50,15,0,255)
*/
#define BOX(x,y,w,h,color,flag) \
		"\n .byte 65, " #x ", " #y ", " #w ", " #h ", " #color ", " #flag "\n"  


/*
 * GFX(X, Y, Width, Hieght, Normal, Mask, Pressed)
 *
 * GUI Object -- Must be declared inside a BEGIN_GUI_SECTION() / END_GUI_SECTION group
 * Draws a Graphical Button at (X,Y) with specified Width and Hieght
 *  [Normal] = Pointer to the Graphic when button IS NOT pressed
 *    [Mask] = Pointer to Button's mask
 * [Pressed] = Pointer to the Graphic when button IS pressed
 *
 * Example:
 * char normal[] = { ... }, mask[] = { ... }, pressed[] = { ... };
 *
 * BEGIN_GUI_SECTION(...)
 *   GFX(0,1,7,5,normal,mask,pressed) 
 *   //...
 */
#define GFX(x,y,w,h,normal,mask,pressed) \
		"\n .byte 2, " #x ", " #y ", " #w ", " #h "\n .even\n .long " #normal ", " #mask ", " #pressed "\n" 

/*
 * END_GUI_SECTION
 * 
 * Termanates a GUI Section declaration (* see BEGIN_GUI_SECTION(...) )
*/
#define END_GUI_SECTION "\n .byte -1\n" );

//********************** GUI Manager Macros **********************


/*
 * BEGIN_GUI_MANAGER(ManagerName,GUIGraphicalVar,NextManager)
 *
 * [ManagerName] = Name to be assigned to the Manager being declared
 * [GUIGraphicalVar] = Pointer to the Graphical Structure controlled by the Manager 
 * 		(usually declared by BEGIN_GUI_SECTION(...) / END_GUI_SECTION
 * [NextManager] = Pointer to the next Manager in the list
 * 
 * ** Must be Terminated by END_GUI_MANAGER
 *
 * Example:
 * 	BEGIN_GUI_MANAGER(GotFocusManager,GotFocus,GUIManager)
 * 	  ON_GUI_EVENT(closeButton,-1)
 * 	  [more events]
 * 	END_GUI_MANAGER
*/
#define BEGIN_GUI_MANAGER(name,guivar,point_to) \
		extern unsigned char name[]; \
		asm( "\n .even\n" #name ": .long " #point_to "\n .word 0\n .byte 127,0\n .long " #guivar "\n"

/*
 * ON_GUI_EVENT(GUIObject, ReturnValue)
 *
 * [GUIObject] = Pointer to Graphical Object which will trigger the event -- usually declared by VARIABLE(name,data)
 * [Return Value] = Value returned when event is triggered (consult the Prosit docs for more on return values)
 *
 * Example:
 * 	ON_GUI_EVENT(dragBar, -1)
*/
#define ON_GUI_EVENT(var,retval) \
		" .long " #var "\n .word " #retval "\n"

/*
 * Terminates a GUI Manager declared by BEGIN_GUI_MANAGER(...) 
 *
 * (* See BEGIN_GUI_MANAGER(...) for more details)
*/
#define END_GUI_MANAGER "\n .long 0\n" );


#endif // PROSIT_MACROS
